"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.default = void 0;

var _objectWithoutPropertiesLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutPropertiesLoose"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _pause = _interopRequireDefault(require("@marcopeg/utils/lib/pause"));

var _serviceLogger = require("@forrestjs/service-logger");

var _conn = require("./conn");

const establishConnection =
/*#__PURE__*/
function () {
  var _ref = (0, _asyncToGenerator2.default)(function* (conn, maxAttempts, attemptDelay) {
    let attempts = 0;
    let lastErrorMSG = '';

    do {
      try {
        (0, _serviceLogger.logVerbose)(`[postgres]  Connection attempt ${attempts + 1}/${maxAttempts} to "${conn.name}"`);
        yield conn.handler.authenticate();
        return true;
      } catch (err) {
        attempts += 1;
        lastErrorMSG = err.message;
        (0, _serviceLogger.logError)(`[postgres] failed connection "${conn.name}" - ${err.message}`);
        (0, _serviceLogger.logDebug)(err);
        yield (0, _pause.default)(attemptDelay);
      }
    } while (attempts < maxAttempts);

    throw new Error(lastErrorMSG);
  });

  return function establishConnection(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

const initModels =
/*#__PURE__*/
function () {
  var _ref2 = (0, _asyncToGenerator2.default)(function* (conn, models) {
    const promises = models.map(model => new Promise(
    /*#__PURE__*/
    function () {
      var _ref3 = (0, _asyncToGenerator2.default)(function* (resolve, reject) {
        try {
          (0, _serviceLogger.logVerbose)(`[posgres] init model "${model.name}" in "${conn.name}"`);
          const instance = yield model.init(conn.handler);
          (0, _conn.pushModel)(conn, instance);
          resolve();
        } catch (err) {
          (0, _serviceLogger.logError)(`[postgres] failed to init model "${model.name}" in "${conn.name}" - ${err.message}`);
          (0, _serviceLogger.logDebug)(err);
          reject({
            model,
            err
          });
        }
      });

      return function (_x6, _x7) {
        return _ref3.apply(this, arguments);
      };
    }()));
    return Promise.all(promises);
  });

  return function initModels(_x4, _x5) {
    return _ref2.apply(this, arguments);
  };
}();

const startModels =
/*#__PURE__*/
function () {
  var _ref4 = (0, _asyncToGenerator2.default)(function* (conn, models) {
    const promises = models.map(model => new Promise(
    /*#__PURE__*/
    function () {
      var _ref5 = (0, _asyncToGenerator2.default)(function* (resolve, reject) {
        if (!model.start) {
          return resolve();
        }

        try {
          (0, _serviceLogger.logVerbose)(`[postgres] start model "${model.name}" in "${conn.name}"`);
          yield model.start(conn, (0, _conn.getModel)(model.name, conn.name));
          resolve();
        } catch (err) {
          (0, _serviceLogger.logError)(`[postgres] failed to start model "${model.name}" in "${conn.name}" - ${err.message}`);
          (0, _serviceLogger.logDebug)(err);
          reject({
            model,
            err
          });
        }
      });

      return function (_x10, _x11) {
        return _ref5.apply(this, arguments);
      };
    }()));
    return Promise.all(promises);
  });

  return function startModels(_x8, _x9) {
    return _ref4.apply(this, arguments);
  };
}();

var _default =
/*#__PURE__*/
function () {
  var _ref6 = (0, _asyncToGenerator2.default)(function* (_ref7) {
    let schemas = _ref7.schemas,
        models = _ref7.models,
        maxAttempts = _ref7.maxAttempts,
        attemptDelay = _ref7.attemptDelay,
        config = (0, _objectWithoutPropertiesLoose2.default)(_ref7, ["schemas", "models", "maxAttempts", "attemptDelay"]);
    (0, _serviceLogger.logInfo)('[postgres] start');
    const name = config.connectionName || 'default';
    const conn = (0, _conn.getHandler)(name);

    try {
      yield establishConnection(conn, maxAttempts, attemptDelay);
    } catch (err) {
      throw new Error(`[postgres] connection failed "${name}" - ${err.message}`);
    }

    try {
      if (config.onConnection) {
        yield config.onConnection(conn);
      }
    } catch (err) {
      throw new Error(`[postgres] onConnection hook failed for "${name}" - ${err.message}`);
    } // Create custom schemas


    if (schemas) {
      const schemasP = schemas.map(schema => conn.handler.query(`CREATE SCHEMA IF NOT EXISTS ${schema};`));
      yield Promise.all(schemasP);
    }

    try {
      yield initModels(conn, models);
    } catch (err) {
      throw new Error(`[postgres] init models failed for "${name}": ${err.model.name}, ${err.err.message}`);
    }

    try {
      if (config.afterInitModels) {
        yield config.afterInitModels(conn);
      }
    } catch (err) {
      throw new Error(`[postgres] afterInitModels hook failed for "${name}" - ${err.message}`);
    }

    try {
      yield startModels(conn, models);
    } catch (err) {
      throw new Error(`[postgres] start models failed for "${name}": ${err.model.name}, ${err.err.message}`);
    }

    try {
      if (config.afterStartModels) {
        yield config.afterStartModels(conn);
      }
    } catch (err) {
      throw new Error(`[postgres] afterStartModels hook failed for "${name}" - ${err.message}`);
    }
  });

  return function (_x12) {
    return _ref6.apply(this, arguments);
  };
}();

exports.default = _default;